require "numru/ggraph"
include NumRu
include NMath

# Earth
Grav    = 9.8
RPlanet = 6.371e6
## Mars
#Grav    = 3.72
#RPlanet = 3.396e6

###############################################################################

def calc_msf( gphys )

gphysout = calc_msf_core( gphys, 'lat', 'level' )

return gphysout

end

###############################################################################

def calc_msf_core( gphys, namelat, namelev )

rmiss = gphys.get_att('missing_value')[0]

gphyscopied = gphys.copy

z = gphyscopied.axis(namelev).pos.convert_units( Units['Pa'] )
z.long_name = 'pressure'
gphyscopied.axis(namelev).set_pos(z)

km    = gphyscopied.shape[1]
lat   = gphyscopied.coord(namelat).val
level = gphyscopied.coord(namelev).val

gphysout = gphyscopied.copy

mask        = gphyscopied.val.get_mask
gphyscopied = gphyscopied.val.set_missing_value(0.0).all_valid


if level[0] > level[1] then
  gphysout[true,km-1] = gphyscopied[true,km-1] * level[km-1] / Grav * 2 * PI * RPlanet * cos( lat[true] * PI / 180.0 )
  k = km-1-1
  while k >= 0
    gphysout[true,k] = gphysout[true,k+1] + ( gphyscopied[true,k] + gphyscopied[true,k+1] ) * 0.5 * ( level[k] - level[k+1] ) / Grav * 2 * PI * RPlanet * cos( lat[true] * PI / 180.0 )
  k -= 1
  end
elsif
  gphysout[true,0] = gphyscopied[true,0] * level[0] / Grav * 2 * PI * RPlanet * cos( lat[true] * PI / 180.0 )
  k = 0+1
  while k < km
    gphysout[true,k] = gphysout[true,k-1] + ( gphyscopied[true,k] + gphyscopied[true,k-1] ) * 0.5 * ( level[k] - level[k-1] ) / Grav * 2 * PI * RPlanet * cos( lat[true] * PI / 180.0 )
  k += 1
  end
end

gphysoutNaMiss = gphysout.val.set_mask(mask)
gphysout[true,true] = gphysoutNaMiss[true,true]

gphysout = gphysout * 1.0e-8

gphysout.long_name = 'mass stream function'
gphysout.units     = '1e8 kg/s'

return gphysout

end

